/*
  Copyright 1999 The University of Rhode Island and The Massachusetts
  Institute of Technology

  Portions of this software were developed by the Graduate School of
  Oceanography (GSO) at the University of Rhode Island (URI) in collaboration
  with The Massachusetts Institute of Technology (MIT).

  Access and use of this software shall impose the following obligations and
  understandings on the user. The user is granted the right, without any fee
  or cost, to use, copy, modify, alter, enhance and distribute this software,
  and any derivative works thereof, and its supporting documentation for any
  purpose whatsoever, provided that this entire notice appears in all copies
  of the software, derivative works and supporting documentation.  Further,
  the user agrees to credit URI/MIT in any publications that result from the
  use of this software or in any product that includes this software. The
  names URI, MIT and/or GSO, however, may not be used in any advertising or
  publicity to endorse or promote any products or commercial entity unless
  specific written permission is obtained from URI/MIT. The user also
  understands that URI/MIT is not obligated to provide the user with any
  support, consulting, training or assistance of any kind with regard to the
  use, operation and performance of this software nor to provide the user
  with any updates, revisions, new versions or "bug fixes".

  THIS SOFTWARE IS PROVIDED BY URI/MIT "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
  EVENT SHALL URI/MIT BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
  DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
  PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTUOUS
  ACTION, ARISING OUT OF OR IN CONNECTION WITH THE ACCESS, USE OR PERFORMANCE
  OF THIS SOFTWARE.
*/

// DODS-netCDF surrogate library functions for data dimensions. See note
// about the symbol LOCAL in Dfile.c
// 
// ReZa 10/6/94

// $Log: Ddim.cc,v $
// Revision 1.1.2.1  1999/10/15 19:50:55  jimg
// Changed return values and conditions for NC API entry points
//
// Revision 1.1  1999/07/28 00:22:41  jimg
// Added
//
// Revision 1.9  1999/05/07 23:45:30  jimg
// String --> string fixes
//
// Revision 1.8  1997/03/25 16:16:28  reza
// Fixed CVS duplicate log reports.
//
// Revision 1.7  1997/02/28 23:32:40  jimg
// Fixed bad assign to rcode in conditional. This was the (in)famous ferret
// bug.
//
// Revision 1.6  1996/09/17 00:26:07  jimg
// Merged changes from a side branch which contained various changes from
// Reza and Charles.
// Removed ncdump and netexec since ncdump is now in its own directory and
// netexec is no longer used.
//
// Revision 1.5  1995/07/10 05:07:40  reza
// added fortran jacket support.
//
// Revision 1.4.2.1  1996/04/02 19:25:04  jimg
// Changed the `conns' variable so that now it is a global *pointer*, not an
// object. Each call that might add to the conns object must first check to see
// that it has been created. This was done because of link-time problems with
// static global objects.
// Fixed up some comment stuff.
// Added assert() calls for conns.
//
// Revision 1.4  1995/07/09  21:33:32  jimg
// Added copyright notice.
//
// Revision 1.3  1994/12/08  19:01:29  reza
// Added support for local netCDF file access through the original netCDF code.
//
// Revision 1.2  1994/11/23  21:05:56  reza
// First working version.
//
// Revision 1.1  1994/11/03  04:35:46  reza
// Preliminary version of netCDF -> DODS library.

#include "config_dap.h"
static char rcsid[] not_used ={"$Id: Ddim.cc,v 1.1.2.1 1999/10/15 19:50:55 jimg Exp $"};

#include <assert.h>
#include "Dnetcdf.h"


int
nc_def_dim(int cdfid, const char *name, size_t size, int *dimidp)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
      rcode = lnc_def_dim((*conns)[cdfid]->ncid(), name, size, dimidp);
      return rcode;
    }
#endif

    return NC_EPERM;
}


int
ncdimdef(int cdfid, const char *name, long size) // version 2 interface
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lncdimdef((*conns)[cdfid]->ncid(), name, size);
	return rcode;
    }
#endif
    cdf_routine_name = "ncdimdef" ;
    NCadvise(NC_EPERM, "DODS network interface is read-only") ;
    return(-1);
}


int
nc_inq_dimid(int cdfid, const char *name, int *dimid_ptr)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
      rcode = lnc_inq_dimid((*conns)[cdfid]->ncid(), name, dimid_ptr);
      return rcode;
    }
#endif

    int dimid = -1;

    for (int i = 0; i < (*conns)[cdfid]->ndims(); i++)
	if ((*conns)[cdfid]->dim_name(i) == name) {
	  dimid = i;
	  break;
	}
  
    if(dimid == -1)
      return NC_EBADDIM;

    *dimid_ptr = dimid;
    return NC_NOERR;
}


int
ncdimid(int cdfid, const char *name) // version 2 interface
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lncdimid((*conns)[cdfid]->ncid(), name);
	return rcode;
    }
#endif
    cdf_routine_name = "ncdimid" ;
  
    rcode = -1;
    for (int i = 0; i < (*conns)[cdfid]->ndims(); i++)
	if ((*conns)[cdfid]->dim_name(i) == name) {
	    rcode = i;
	    break;
	}
  
    if(rcode == -1)
	NCadvise(NC_EBADDIM, "dim \"%s\" not found", name) ;

    return rcode;
}


int
nc_inq_dim(int cdfid, int dimid, char *name, size_t *sizep)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
      rcode = lnc_inq_dim((*conns)[cdfid]->ncid(), dimid, name, sizep);
      return rcode;
    }
#endif

    if(( dimid < 0) || (dimid > (*conns)[cdfid]->ndims())) 
      return NC_EBADDIM ;

    if (name != NULL) 
	(void) strcpy (name, (*conns)[cdfid]->dim_name(dimid).c_str());

    if(sizep != 0) 
	*sizep = (size_t) (*conns)[cdfid]->dim_size(dimid); 
    
    return NC_NOERR;
}


int 
nc_inq_dimname(int cdfid, int dimid, char *name)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
      rcode = lnc_inq_dimname((*conns)[cdfid]->ncid(), dimid, name);
      return rcode;
    }
#endif

    if(( dimid < 0) || (dimid > (*conns)[cdfid]->ndims())) 
      return NC_EBADDIM ;

    if (name != NULL) 
	(void) strcpy (name, (*conns)[cdfid]->dim_name(dimid).c_str());

    return NC_NOERR;
}


int 
nc_inq_dimlen(int cdfid, int dimid, size_t *lenp)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
      rcode = lnc_inq_dimlen((*conns)[cdfid]->ncid(), dimid, lenp);
      return rcode;
    }
#endif

    if(( dimid < 0) || (dimid > (*conns)[cdfid]->ndims())) 
      return NC_EBADDIM ;

    if(lenp != 0) 
	*lenp = (size_t) (*conns)[cdfid]->dim_size(dimid); 
    
    return NC_NOERR;
}


int
ncdiminq( int cdfid, int dimid, char *name, long *sizep) // version 2 interface
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lncdiminq((*conns)[cdfid]->ncid(), dimid, name, sizep);
	return rcode;
    }
#endif
    cdf_routine_name = "ncdiminq" ;

    if(( dimid < 0) || (dimid > (*conns)[cdfid]->ndims())) {
	NCadvise(NC_EBADDIM, "%d is not a valid dimension id", dimid) ;
	return(-1) ;
    }

    rcode = dimid;

    if (name != NULL) {
	(void) strcpy (name, (*conns)[cdfid]->dim_name(dimid).c_str());
    }

    if(sizep != 0) {
	*sizep = (long) (*conns)[cdfid]->dim_size(dimid); 
    }

    return rcode;
}


int
nc_rename_dim(int cdfid, int dimid, const char *newname)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
      rcode = lnc_rename_dim((*conns)[cdfid]->ncid(), dimid, newname);
      return rcode;
    }
#endif

    return NC_EPERM;
}


int
ncdimrename(int cdfid, int dimid,const char *newname) // version 2 interface
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lncdimrename((*conns)[cdfid]->ncid(), dimid, newname);
	return rcode;
    }
#endif
    cdf_routine_name = "ncdimrename" ;
    NCadvise(NC_EPERM, "DODS network interface is read-only") ;
    return(-1);
}
